==> ask-cli-1.1.1.tgz: package/lib/new/git-templates.js <==
'use strict';

const request = require('request');
const git = require('simple-git');
const inquirer = require('inquirer');
const tools = require('../utils/tools');
const Spinner = require('clui').Spinner;
const nodeModuleInstall = require('../utils/install-node-module');
const path = require('path');
const jsonUtility = require('../utils/json-utility');
const skillParser = require('../utils/skill-parser');
const jsonfile = require('jsonfile');

module.exports = {
    clone: clone
};

function clone(url, skillName, templateName, callback) {
    getTemplateMap(url, (err, templateMap) => {
        if (err) {
            callback(err);
        } else {
            let templateList = Object.keys(templateMap);
            getTargetTemplateName(
                templateList,
                templateName,
                (err, templateName) => {
                    if (err) {
                        callback(err);
                    } else {
                        let cloneUrl = templateMap[templateName].gitUrl;
                        skillName = skillName || templateName;
                        let validSkillName = preprocessPackageName(skillName);
                        gitCloneTemplate(
                            cloneUrl,
                            validSkillName,
                            callback);
                    }
                }
            );
        }
    });
}

// Private
function getTemplateMap(url, callback) {
    let S3_ENDPOINT = url;
    let headers = {};
    headers['User-Agent'] = 'ask-cli/' + require('../../package.json').version +
        ' Node/' + process.version;
    let params = {
        url: S3_ENDPOINT,
        method: 'GET',
        headers: headers
    };
    request(params, (error, response) => {
        if (error || response.statusCode === null) {
            let error = 'Cannot retrieve ASK template list from S3 bucket.';
            callback(error);
        } else {
            let templatesJSONMap = tools.convertDataToJsonObject(response.body);
            callback(null, templatesJSONMap);
        }
    });
}


function getTargetTemplateName(templateList, inputName, callback) {
    if (inputName) {
        if (templateList.indexOf(inputName) === -1) {
            let error = '[Error]: no template associate with the input name';
            callback(error);
        } else {
            callback(null, inputName);
        }
    } else {
        let templateChoiceQuestion = {
            type: 'list',
            message: 'List of templates you can chose',
            name: 'templateName',
            choices: templateList.sort()
        };

        inquirer.prompt([templateChoiceQuestion])
                .then((answer) => {
                    let templateName = answer.templateName;
                    callback(null, templateName);
                });
    }
}

function gitCloneTemplate(cloneUrl, skillPackageName, callback) {
    let installSpinner = new Spinner(' Initializing Alexa skill project from the chosen' +
        ' template...');
    installSpinner.start();
    let cloneDir = path.join(process.cwd(), skillPackageName);
    git().silent(true)
        .clone(cloneUrl, cloneDir, (err) => {
            if (err) {
                installSpinner.stop();
                callback(err);
            }
            else {
                let skillFilePath = path.join(cloneDir, 'skill.json');
                let manifest = jsonUtility.read(skillFilePath);
                let skillInfo = skillParser.extractSkillInfo(manifest);
                let configPath = path.join(cloneDir, '.ask/config');

                nodeModuleInstall.install(skillInfo, cloneDir, (err) => {
                    changeSkillName(skillPackageName, skillFilePath);
                    changeLambdaName(skillPackageName, configPath);
                    installSpinner.stop();
                    if (err) callback(err);
                    else callback(null, skillPackageName);
                });
            }
        });
}

function preprocessPackageName(name) {
    // return a name without space or another weird stuff. so that it can be read.
    return name.toLowerCase().replace(/[\W_]+/g,'-');
}

function changeSkillName(skillName, skillPath) {
    let manifest = jsonfile.readFileSync(skillPath);
    let locales = jsonUtility.getPropertyFromJsonObject(manifest,
        ['manifest', 'publishingInformation', 'locales']);
    if (!locales) {
        locales = jsonUtility.getPropertyFromJsonObject(manifest,
            ['skillManifest', 'publishingInformation', 'locales']);
    }
    Object.keys(locales).forEach(locale => {
        locales[locale].name = skillName;
    });
    jsonfile.writeFileSync(skillPath, manifest, {spaces: 2});
}

// TODO: accept different profile. make a new function copy the config from one profile to
// TODO: another one. then change the name.
function changeLambdaName(lambdaName, configPath) {
    let config = jsonfile.readFileSync(configPath);

    // template profile should be default as "default"
    let apis = jsonUtility.getPropertyFromJsonObject(config,
        ['deploy_settings', 'default', 'merge', 'manifest', 'apis']);
    if (!apis) {
        apis = jsonUtility.getPropertyFromJsonObject(config,
            ['deploy_settings', 'default', 'merge', 'skillManifest', 'apis']);
    }


    for (let domain of Object.keys(apis)) {
        for (let key of Object.keys(apis[domain])) {
            if (!apis[domain][key]) {
                continue;
            }
            if (key === 'endpoint') {
                if (apis[domain][key].uri) {
                    apis[domain][key].uri = "ask-" + domain + "-" + lambdaName + "-default";
                }
            }
            // if it's not endpoint, it has to be region.
            if (key === 'regions') {
                for (let region of Object.keys(apis[domain][key])) {
                    if (apis[domain][key][region].endpoint
                        || apis[domain][key][region].endpoint.uri) {
                        apis[domain][key][region].endpoint.uri = "ask-" + domain + "-" + lambdaName + "-default";
                    }
                }
            }
        }
    }
    jsonfile.writeFileSync(configPath, config, {spaces: 2});
}

==> ask-cli-1.2.0.tgz: package/lib/new/git-templates.js <==
'use strict';

const request = require('request');
const git = require('simple-git');
const inquirer = require('inquirer');
const tools = require('../utils/tools');
const Spinner = require('clui').Spinner;
const nodeModuleInstall = require('../utils/install-node-module');
const path = require('path');
const jsonUtility = require('../utils/json-utility');
const skillParser = require('../utils/skill-parser');
const jsonfile = require('jsonfile');

module.exports = {
    clone: clone
};

function clone(url, skillName, templateName, callback) {
    getTemplateMap(url, (err, templateMap) => {
        if (err) {
            callback(err);
        } else {
            if (!templateMap) {
                callback('Cannot retrieve template list.');
                return;
            }
            let templateList = Object.keys(templateMap);
            getTargetTemplateName(
                templateList,
                templateName,
                (err, templateName) => {
                    if (err) {
                        callback(err);
                    } else {
                        let cloneUrl = templateMap[templateName].gitUrl;
                        skillName = skillName || templateName;
                        let validSkillName = preprocessPackageName(skillName);
                        gitCloneTemplate(
                            cloneUrl,
                            validSkillName,
                            callback);
                    }
                }
            );
        }
    });
}

// Private
function getTemplateMap(url, callback) {
    let S3_ENDPOINT = url;
    let headers = {};
    headers['User-Agent'] = 'ask-cli/' + require('../../package.json').version +
        ' Node/' + process.version;
    let params = {
        url: S3_ENDPOINT,
        method: 'GET',
        headers: headers
    };
    request(params, (error, response) => {
        if (error || response.statusCode === null) {
            let error = 'Cannot retrieve template list.';
            callback(error);
        } else {
            let templatesJSONMap = tools.convertDataToJsonObject(response.body);
            callback(null, templatesJSONMap);
        }
    });
}


function getTargetTemplateName(templateList, inputName, callback) {
    if (inputName) {
        if (templateList.indexOf(inputName) === -1) {
            let error = '[Error]: no template associate with the input name';
            callback(error);
        } else {
            callback(null, inputName);
        }
    } else {
        let templateChoiceQuestion = {
            type: 'list',
            message: 'List of templates you can chose',
            name: 'templateName',
            choices: templateList.sort()
        };

        inquirer.prompt([templateChoiceQuestion])
                .then((answer) => {
                    let templateName = answer.templateName;
                    callback(null, templateName);
                });
    }
}

function gitCloneTemplate(cloneUrl, skillPackageName, callback) {
    let installSpinner = new Spinner(' Initializing Alexa skill project from the chosen' +
        ' template...');
    installSpinner.start();
    let cloneDir = path.join(process.cwd(), skillPackageName);
    git().silent(true)
        .clone(cloneUrl, cloneDir, (err) => {
            if (err) {
                installSpinner.stop();
                callback(err);
            }
            else {
                let skillFilePath = path.join(cloneDir, 'skill.json');
                let manifest = jsonUtility.read(skillFilePath);
                let skillInfo = skillParser.extractSkillInfo(manifest);
                let configPath = path.join(cloneDir, '.ask/config');

                nodeModuleInstall.install(skillInfo, cloneDir, (err) => {
                    changeSkillName(skillPackageName, skillFilePath);
                    changeLambdaName(skillPackageName, configPath);
                    installSpinner.stop();
                    if (err) callback(err);
                    else callback(null, skillPackageName);
                });
            }
        });
}

function preprocessPackageName(name) {
    // return a name without space or another weird stuff. so that it can be read.
    return name.toLowerCase().replace(/[\W_]+/g,'-');
}

function changeSkillName(skillName, skillPath) {
    let manifest = jsonfile.readFileSync(skillPath);
    let locales = jsonUtility.getPropertyFromJsonObject(manifest,
        ['manifest', 'publishingInformation', 'locales']);
    if (!locales) {
        locales = jsonUtility.getPropertyFromJsonObject(manifest,
            ['skillManifest', 'publishingInformation', 'locales']);
    }
    Object.keys(locales).forEach(locale => {
        locales[locale].name = skillName;
    });
    jsonfile.writeFileSync(skillPath, manifest, {spaces: 2});
}

// TODO: accept different profile. make a new function copy the config from one profile to
// TODO: another one. then change the name.
function changeLambdaName(lambdaName, configPath) {
    let config = jsonfile.readFileSync(configPath);

    // template profile should be default as "default"
    let apis = jsonUtility.getPropertyFromJsonObject(config,
        ['deploy_settings', 'default', 'merge', 'manifest', 'apis']);
    if (!apis) {
        apis = jsonUtility.getPropertyFromJsonObject(config,
            ['deploy_settings', 'default', 'merge', 'skillManifest', 'apis']);
    }


    for (let domain of Object.keys(apis)) {
        for (let key of Object.keys(apis[domain])) {
            if (!apis[domain][key]) {
                continue;
            }
            if (key === 'endpoint') {
                if (apis[domain][key].uri) {
                    apis[domain][key].uri = "ask-" + domain + "-" + lambdaName + "-default";
                }
            }
            // if it's not endpoint, it has to be region.
            if (key === 'regions') {
                for (let region of Object.keys(apis[domain][key])) {
                    if (apis[domain][key][region].endpoint
                        || apis[domain][key][region].endpoint.uri) {
                        apis[domain][key][region].endpoint.uri = "ask-" + domain + "-" + lambdaName + "-default";
                    }
                }
            }
        }
    }
    jsonfile.writeFileSync(configPath, config, {spaces: 2});
}
